mutringbuf::iterators::sync_iterators::work_iter

Struct WorkIter

pub struct WorkIter<'buf, B: MutRB> { /* private fields */ }
Iterator used to mutate elements in-place.

This iterator returns mutable references to data stored within the buffer. Thus, as stated in the docs below, Self::advance has to be called when done with the mutation in order to move the iterator.

Self::advance updates a global iterator, which is read by the consumer to decide if it can move on. To avoid this Detached can be obtained by calling Self::detach.

Implementations

Source

impl<'buf, B: MutRB<Item = T>, T> WorkIter<'buf, B>

pub fn reset_index(&mut self)

Resets the index of the iterator. I.e., moves the iterator to the location occupied by its successor.