pub struct WorkIter<'buf, B: MutRB> { /* private fields */ }Expand description
Iterator used to mutate elements in-place.
This iterator returns mutable references to data stored within the buffer.
Thus, as stated in the docs below, Self::advance has to be called when done with the mutation
in order to move the iterator.
Self::advance updates a global iterator, which is read by the consumer to decide if it can move on.
To avoid this Detached can be obtained by calling Self::detach.
Implementations